/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
/*
 * Copyright (c) 2019 Amlogic, Inc. All rights reserved.
 */

#ifndef _HDMI_TX_REG_H
#define _HDMI_TX_REG_H

/* TOP-level wrapper registers addresses
 * bit24: 1 means secure access
 * bit28: 1 means DWC, 0 means TOP
 */
#define TOP_OFFSET_MASK      (0x1UL << 24)
#define SEC_OFFSET           (0x1UL << 25)
#define TOP_SEC_OFFSET_MASK  ((TOP_OFFSET_MASK) | (SEC_OFFSET))

#define HDMITX_TOP_SW_RESET           (TOP_OFFSET_MASK + (0x000 << 2)) // 0x000
#define HDMITX_TOP_CLK_CNTL           (TOP_OFFSET_MASK + (0x001 << 2)) // 0x004
#define HDMITX_TOP_HPD_FILTER         (TOP_OFFSET_MASK + (0x002 << 2)) // 0x008
#define HDMITX_TOP_INTR_MASKN         (TOP_OFFSET_MASK + (0x003 << 2)) // 0x00C
#define HDMITX_TOP_INTR_STAT          (TOP_OFFSET_MASK + (0x004 << 2)) // 0x010
#define HDMITX_TOP_INTR_STAT_CLR      (TOP_OFFSET_MASK + (0x005 << 2)) // 0x014
#define HDMITX_TOP_BIST_CNTL          (TOP_OFFSET_MASK + (0x006 << 2)) // 0x018
#define HDMITX_TOP_SHIFT_PTTN_012     (TOP_OFFSET_MASK + (0x007 << 2)) // 0x01C
#define HDMITX_TOP_SHIFT_PTTN_345     (TOP_OFFSET_MASK + (0x008 << 2)) // 0x020
#define HDMITX_TOP_SHIFT_PTTN_67      (TOP_OFFSET_MASK + (0x009 << 2)) // 0x024
#define HDMITX_TOP_TMDS_CLK_PTTN_01   (TOP_OFFSET_MASK + (0x00A << 2)) // 0x028
#define HDMITX_TOP_TMDS_CLK_PTTN_23   (TOP_OFFSET_MASK + (0x00B << 2)) // 0x02C
#define HDMITX_TOP_TMDS_CLK_PTTN_CNTL (TOP_OFFSET_MASK + (0x00C << 2)) // 0x030
#define HDMITX_TOP_STAT0              (TOP_OFFSET_MASK + (0x00E << 2)) // 0x038
#define HDMITX_TOP_SKP_CNTL_STAT      (TOP_OFFSET_MASK + (0x010 << 2)) // 0x040
#define HDMITX_TOP_INFILTER           (TOP_OFFSET_MASK + (0x01D << 2)) // 0x074
#define HDMITX_TOP_NSEC_SCRATCH       (TOP_OFFSET_MASK + (0x01E << 2)) // 0x078
#define HDMITX_TOP_SEC_SCRATCH        (TOP_SEC_OFFSET_MASK + (0x01F << 2)) // 0x07C
#define HDMITX_TOP_HS_INTR_CNTL       (TOP_OFFSET_MASK + (0x021 << 2)) // 0x084
#define HDMITX_TOP_AUD_ID             (TOP_OFFSET_MASK + (0x022 << 2)) // 0x088
#define HDMITX_TOP_HV_ACTIVE          (TOP_OFFSET_MASK + (0x023 << 2)) // 0x08C
#define HDMITX_TOP_I2C_BUSY_CNT_MAX   (TOP_OFFSET_MASK + (0x028 << 2)) // 0x0A0
#define HDMITX_TOP_I2C_BUSY_CNT_STAT  (TOP_OFFSET_MASK + (0x029 << 2)) // 0x0A4
#define HDMITX_TOP_HDCP22_BSOD        (TOP_OFFSET_MASK + (0x02A << 2)) // 0x0A8
#define HDMITX_TOP_DDC_CNTL           (TOP_OFFSET_MASK + (0x02B << 2)) // 0x0AC
#define HDMITX_TOP_FIX_DISABLE_0      (TOP_OFFSET_MASK + (0x02C << 2)) // 0x0B0
#define HDMITX_TOP_FIX_DISABLE_1      (TOP_OFFSET_MASK + (0x02D << 2)) // 0x0B4
#define HDMITX_TOP_DISABLE_NULL       (TOP_OFFSET_MASK + (0x030 << 2)) // 0x0C0
#define HDMITX_TOP_MISC_CNTL          (TOP_OFFSET_MASK + (0x032 << 2)) // 0x0C8
#define HDMITX_TOP_SEC_VIDEO_OVR      (TOP_OFFSET_MASK + (0x033 << 2)) // 0x0CC
#define HDMITX_TOP_HDCP14_MIN_SIZE    (TOP_SEC_OFFSET_MASK + (0x034 << 2)) // 0x0D0
#define HDMITX_TOP_HDCP22_MIN_SIZE    (TOP_SEC_OFFSET_MASK + (0x035 << 2)) // 0x0D4
#define HDMITX_TOP_SECURE_INDEX       (TOP_SEC_OFFSET_MASK + (0x040 << 2)) // 0x100
#define HDMITX_TOP_SECURE_DATA        (TOP_SEC_OFFSET_MASK + (0x041 << 2)) // 0x104
#define HDMITX_TOP_DONT_TOUCH0        (TOP_SEC_OFFSET_MASK + (0x0FE << 2)) // 0x3F8
#define HDMITX_TOP_DONT_TOUCH1        (TOP_SEC_OFFSET_MASK + (0x0FF << 2)) // 0x3FC

//------------------------------------------------------------------------------
// INVECAS TX CONTROLLER REGISTERS ADDRESSES
//------------------------------------------------------------------------------

//====================AON_DDC_REG===============
#define            INTR3_IVCTX 0x00000000
#define       INTR3_MASK_IVCTX 0x00000001
#define       DDC_MANUAL_IVCTX 0x00000002
#define         DDC_ADDR_IVCTX 0x00000003
#define         DDC_SEGM_IVCTX 0x00000004
#define       DDC_OFFSET_IVCTX 0x00000005
#define     DDC_DIN_CNT1_IVCTX 0x00000006
#define     DDC_DIN_CNT2_IVCTX 0x00000007
#define       DDC_STATUS_IVCTX 0x00000008
#define          DDC_CMD_IVCTX 0x00000009
#define     DDC_DATA_AON_IVCTX 0x0000000A
#define     DDC_DOUT_CNT_IVCTX 0x0000000B
#define    DDC_DELAY_CNT_IVCTX 0x0000000C
#define DDC_TIMEOUT_CNT1_IVCTX 0x0000000D
#define DDC_TIMEOUT_CNT2_IVCTX 0x0000000E
#define DDC_TIMEOUT_CNT3_IVCTX 0x0000000F
#define  DDC_TIMEOUT_CTL_IVCTX 0x00000010
#define     TPI_HPD_RSEN_IVCTX 0x00000011
#define         HTPLG_T2_IVCTX 0x00000012
#define         HTPLG_T1_IVCTX 0x00000013
#define         HPD_CTRL_IVCTX 0x00000014
#define      HPD_IN_CTRL_IVCTX 0x00000015
#define            INTR1_IVCTX 0x00000016
#define       INTR1_MASK_IVCTX 0x00000017
#define      AON_CYP_CTL_IVCTX 0x00000018

//====================SYSSOC_P0_REG===============
#define             VND_IDL_IVCTX 0x00000100
#define             VND_IDH_IVCTX 0x00000101
#define             DEV_IDL_IVCTX 0x00000102
#define             DEV_IDH_IVCTX 0x00000103
#define            DEV_REVL_IVCTX 0x00000104
#define            DEV_REVH_IVCTX 0x00000105
#define            SOC_INTR_IVCTX 0x00000106
#define        SOC_INTRMASK_IVCTX 0x00000107
#define          HOST_CTRL1_IVCTX 0x00000108
#define          HOST_CTRL2_IVCTX 0x00000109
#define        OTP_DBYTE510_IVCTX 0x0000010A
#define        SOC_FUNC_SEL_IVCTX 0x0000010B
#define           SYS_CTRL1_IVCTX 0x0000010C
#define              CLKPWD_IVCTX 0x0000010D
#define           CLKPHASE1_IVCTX 0x0000010E
#define           CLKPHASE2_IVCTX 0x0000010F
#define            PWD_SRST_IVCTX 0x00000110
#define             SW_RST2_IVCTX 0x00000111
#define             SW_RST3_IVCTX 0x00000112
#define            SYS_MISC_IVCTX 0x00000113
#define           SYS_MISC2_IVCTX 0x00000114
#define           SYS_MISC3_IVCTX 0x00000115
#define            PRIFCNTL_IVCTX 0x00000116
#define         PAGE_1_ADDR_IVCTX 0x00000117
#define         PAGE_2_ADDR_IVCTX 0x00000118
#define         PAGE_3_ADDR_IVCTX 0x00000119
#define   PAGE_MHLSPEC_ADDR_IVCTX 0x0000011A
#define      PAGE_CBUS_ADDR_IVCTX 0x0000011B
#define         HW_TPI_ADDR_IVCTX 0x0000011C
#define         PAGE_7_ADDR_IVCTX 0x0000011D
#define         PAGE_8_ADDR_IVCTX 0x0000011E
#define         PAGE_9_ADDR_IVCTX 0x0000011F
#define        PAGE_10_ADDR_IVCTX 0x00000120
#define        PAGE_11_ADDR_IVCTX 0x00000121
#define        PAGE_12_ADDR_IVCTX 0x00000122
#define        PAGE_13_ADDR_IVCTX 0x00000123
#define        PAGE_14_ADDR_IVCTX 0x00000124
#define        PAGE_15_ADDR_IVCTX 0x00000125
#define           IPREGRST0_IVCTX 0x00000126
#define           IPREGRST1_IVCTX 0x00000127
#define           IPREGRST2_IVCTX 0x00000128
#define           IPREGRST3_IVCTX 0x00000129
#define     PCLK2TMDS_MISC0_IVCTX 0x0000012A
#define     PCLK2TMDS_MISC1_IVCTX 0x0000012B
#define      HDCP2X_CONTROL_IVCTX 0x0000012C
#define      VIDGEN_CONTROL_IVCTX 0x0000012D
#define VP_BUF_ALIGN_STATUS_IVCTX 0x0000012E
#define       CLK_DIV_CNTRL_IVCTX 0x0000012F
//====================SYSTOP_REG===============
#define            TOP_INTR_IVCTX 0x00000130

//====================SW_TPI_REG===============
#define           DEBUG_MODE_EN_IVCTX 0x00000200
#define                SYS_STAT_IVCTX 0x00000201
#define               SYS_CTRL3_IVCTX 0x00000202
#define                     DPD_IVCTX 0x00000203
#define               SYS_CTRL4_IVCTX 0x00000204
#define                 MCUTOUT_IVCTX 0x00000205
#define            DIGIPREGRST0_IVCTX 0x00000206
#define            DIGIPREGRST1_IVCTX 0x00000207
#define            DIGIPREGRST2_IVCTX 0x00000208
#define            DIGIPREGRST3_IVCTX 0x00000209
#define           HDCP_CTRL_0_0_IVCTX 0x0000020E
#define               HDCP_CTRL_IVCTX 0x0000020F
#define               WR_BKSV_1_IVCTX 0x00000210
#define               WR_BKSV_2_IVCTX 0x00000211
#define               WR_BKSV_3_IVCTX 0x00000212
#define               WR_BKSV_4_IVCTX 0x00000213
#define               WR_BKSV_5_IVCTX 0x00000214
#define                     AN1_IVCTX 0x00000215
#define                     AN2_IVCTX 0x00000216
#define                     AN3_IVCTX 0x00000217
#define                     AN4_IVCTX 0x00000218
#define                     AN5_IVCTX 0x00000219
#define                     AN6_IVCTX 0x0000021A
#define                     AN7_IVCTX 0x0000021B
#define                     AN8_IVCTX 0x0000021C
#define                  AKSV_1_IVCTX 0x0000021D
#define                  AKSV_2_IVCTX 0x0000021E
#define                  AKSV_3_IVCTX 0x0000021F
#define                  AKSV_4_IVCTX 0x00000220
#define                  AKSV_5_IVCTX 0x00000221
#define                    RI_1_IVCTX 0x00000222
#define                    RI_2_IVCTX 0x00000223
#define             RI_128_COMP_IVCTX 0x00000224
#define              HDCP_I_CNT_IVCTX 0x00000225
#define                 RI_STAT_IVCTX 0x00000226
#define                  RI_CMD_IVCTX 0x00000227
#define                RI_START_IVCTX 0x00000228
#define                 RI_RX_1_IVCTX 0x00000229
#define                 RI_RX_2_IVCTX 0x0000022A
#define            TXHDCP_DEBUG_IVCTX 0x0000022B
#define            VSYNC_START0_IVCTX 0x00000230
#define            VSYNC_START1_IVCTX 0x00000231
#define              VSYNC_END0_IVCTX 0x00000232
#define              VSYNC_END1_IVCTX 0x00000233
#define                CLKRATIO_IVCTX 0x00000235
#define                P2T_CTRL_IVCTX 0x00000236
#define              VID_BLANK0_IVCTX 0x00000237
#define              VID_BLANK1_IVCTX 0x00000238
#define              VID_BLANK2_IVCTX 0x00000239
#define               VID_OVRRD_IVCTX 0x0000023A
#define               AP_DEPROC_IVCTX 0x0000023B
#define    HBLANK_REKEY_CONTROL_IVCTX 0x0000023C
#define              HDPLL_FIX0_IVCTX 0x0000023D
#define              HDPLL_FIX1_IVCTX 0x0000023E
#define              POL_DETECT_IVCTX 0x0000023F
#define              HDPLL_FIX2_IVCTX 0x00000240
#define              HDPLL_FIX3_IVCTX 0x00000241
#define       VSI_MHL_IEEE_NO_0_IVCTX 0x00000242
#define       VSI_MHL_IEEE_NO_1_IVCTX 0x00000243
#define       VSI_MHL_IEEE_NO_2_IVCTX 0x00000244
#define            PKT_FILTER_0_IVCTX 0x00000245
#define            PKT_FILTER_1_IVCTX 0x00000246
#define            PKT_FILTER_2_IVCTX 0x00000247
#define            PKT_FILTER_3_IVCTX 0x00000248
#define         PKT_FILTER_CTRL_IVCTX 0x00000249
#define         DROP_GEN_TYPE_0_IVCTX 0x0000024A
#define         DROP_GEN_TYPE_1_IVCTX 0x0000024B
#define         DROP_D_HDR_TYPE_IVCTX 0x0000024C
#define               DIPT_CNTL_IVCTX 0x0000024D
#define            D_HDR_ORG_ID_IVCTX 0x0000024E
#define              SYS_CNTR_0_IVCTX 0x0000024F
#define              SYS_CNTR_1_IVCTX 0x00000250
#define            SYS_CNTR_ST0_IVCTX 0x00000251
#define            SYS_CNTR_ST1_IVCTX 0x00000252
#define          L1_INTR_STAT_0_IVCTX 0x00000253
#define          L1_INTR_STAT_1_IVCTX 0x00000254
#define         HTPLG_T2_SW_TPI_IVCTX 0x0000025F
#define         HTPLG_T1_SW_TPI_IVCTX 0x00000260
#define              INTR_STATE_IVCTX 0x00000263
#define            INTR1_SW_TPI_IVCTX 0x00000264
#define            INTR2_SW_TPI_IVCTX 0x00000265
#define            INTR3_SW_TPI_IVCTX 0x00000266
#define            INTR5_SW_TPI_IVCTX 0x00000267
#define            RSVD4_SW_TPI_IVCTX 0x00000268
#define            RSVD5_SW_TPI_IVCTX 0x00000269
#define       INTR1_MASK_SW_TPI_IVCTX 0x0000026A
#define       INTR2_MASK_SW_TPI_IVCTX 0x0000026B
#define       INTR3_MASK_SW_TPI_IVCTX 0x0000026C
#define       INTR5_MASK_SW_TPI_IVCTX 0x0000026D
#define            RSVD6_SW_TPI_IVCTX 0x0000026E
#define            RSVD7_SW_TPI_IVCTX 0x0000026F
#define         HPD_CTRL_SW_TPI_IVCTX 0x00000270
#define            RSVD9_SW_TPI_IVCTX 0x00000273
#define              TMDS_CSTAT_IVCTX 0x00000274
#define           RSVD10_SW_TPI_IVCTX 0x00000275
#define           RSVD11_SW_TPI_IVCTX 0x00000276
#define            GCP_PKT_CTRL_IVCTX 0x00000277
#define              TMDS_CTRL4_IVCTX 0x00000278
#define        SCDT_HOLDOFF_MSB_IVCTX 0x00000279
#define              TMDS_CTRL7_IVCTX 0x0000027B
#define     HPD_IN_CTRL_SW_TPI_IVCTX  0x0000027C
#define          VSIF_IEEE_ID_0_IVCTX 0x0000027D
#define          VSIF_IEEE_ID_1_IVCTX 0x0000027E
#define          VSIF_IEEE_ID_2_IVCTX 0x0000027F
#define     DROP_CVTEM_PKT_TYPE_IVCTX 0x00000280
#define      DROP_VTEM_PKT_TYPE_IVCTX 0x00000281
#define      DROP_VSEM_PKT_TYPE_IVCTX 0x00000282
#define             VSEM_ORG_ID_IVCTX 0x00000283
#define        DSC_CVTEM_ORG_ID_IVCTX 0x00000284
#define             VTEM_ORG_ID_IVCTX 0x00000285
#define      DSC_CVTEM_DS_TAG_1_IVCTX 0x00000286
#define      DSC_CVTEM_DS_TAG_2_IVCTX 0x00000287
#define           VTEM_DS_TAG_1_IVCTX 0x00000288
#define           VTEM_DS_TAG_2_IVCTX 0x00000289
#define           FAPA_PKT_CNTL_IVCTX 0x0000028A
#define          VSIF2_PKT_CTRL_IVCTX 0x0000028B
#define          VSIF2_PKT_TYPE_IVCTX 0x0000028C
#define       VSIF2_PKT_VERSION_IVCTX 0x0000028D
#define      VSIF2_PKT_IEEE_ID0_IVCTX 0x0000028E
#define      VSIF2_PKT_IEEE_ID1_IVCTX 0x0000028F
#define      VSIF2_PKT_IEEE_ID2_IVCTX 0x00000290
#define      DROP_D_HDR_PC_TYPE_IVCTX 0x00000291
#define        DROP_DSC_PC_TYPE_IVCTX 0x00000292
#define    D_HDR_PC_PKT_VERSION_IVCTX 0x00000293
#define      DSC_PC_PKT_VERSION_IVCTX 0x00000294
#define        HDCP_MISC_CTRL_0_IVCTX 0x00000295
#define        HDCP_MISC_CTRL_1_IVCTX 0x00000296
#define        EMP_ARBITER_CTRL_IVCTX 0x00000297
#define      EMP_ARBITER_CTRL_1_IVCTX 0x00000298
#define        EMP_PKT_TYPE_VAL_IVCTX 0x00000299
#define         HDPLL_FIX_NEW_0_IVCTX 0x0000029A
#define         HDPLL_FIX_NEW_1_IVCTX 0x0000029B
#define         HDPLL_FIX_NEW_2_IVCTX 0x0000029C
#define    FRL_LINK_RATE_CONFIG_IVCTX 0x0000029D
#define         FAPA_PKT_HEADER_IVCTX 0x0000029E
#define                  LM_DDC_IVCTX 0x0000029F
#define              TXSHA_CTRL_IVCTX 0x000002A4
#define              TXKSV_FIFO_IVCTX 0x000002A5
#define           TXDS_BSTATUS1_IVCTX 0x000002A6
#define           TXDS_BSTATUS2_IVCTX 0x000002A7
#define                 TXVH0_0_IVCTX 0x000002AB
#define                 TXVH0_1_IVCTX 0x000002AC
#define                 TXVH0_2_IVCTX 0x000002AD
#define                 TXVH0_3_IVCTX 0x000002AE
#define                 TXVH1_0_IVCTX 0x000002AF
#define                 TXVH1_1_IVCTX 0x000002B0
#define                 TXVH1_2_IVCTX 0x000002B1
#define                 TXVH1_3_IVCTX 0x000002B2
#define                 TXVH2_0_IVCTX 0x000002B3
#define                 TXVH2_1_IVCTX 0x000002B4
#define                 TXVH2_2_IVCTX 0x000002B5
#define                 TXVH2_3_IVCTX 0x000002B6
#define                 TXVH3_0_IVCTX 0x000002B7
#define                 TXVH3_1_IVCTX 0x000002B8
#define                 TXVH3_2_IVCTX 0x000002B9
#define                 TXVH3_3_IVCTX 0x000002BA
#define                 TXVH4_0_IVCTX 0x000002BB
#define                 TXVH4_1_IVCTX 0x000002BC
#define                 TXVH4_2_IVCTX 0x000002BD
#define                 TXVH4_3_IVCTX 0x000002BE
#define       DDC_MANUAL_SW_TPI_IVCTX 0x000002BF
#define         DDC_ADDR_SW_TPI_IVCTX 0x000002C0
#define         DDC_SEGM_SW_TPI_IVCTX 0x000002C1
#define       DDC_OFFSET_SW_TPI_IVCTX 0x000002C2
#define     DDC_DIN_CNT1_SW_TPI_IVCTX 0x000002C3
#define     DDC_DIN_CNT2_SW_TPI_IVCTX 0x000002C4
#define       DDC_STATUS_SW_TPI_IVCTX 0x000002C5
#define          DDC_CMD_SW_TPI_IVCTX 0x000002C6
#define                DDC_DATA_IVCTX 0x000002C7
#define     DDC_DOUT_CNT_SW_TPI_IVCTX 0x000002C8
#define    DDC_DELAY_CNT_SW_TPI_IVCTX 0x000002C9
#define             TEST_TXCTRL_IVCTX 0x000002CA
#define                    EPST_IVCTX 0x000002CB
#define                    EPCM_IVCTX 0x000002CC
#define                 OTP_CLK_IVCTX 0x000002CD
#define DDC_TIMEOUT_CNT1_SW_TPI_IVCTX 0x000002CE
#define DDC_TIMEOUT_CNT2_SW_TPI_IVCTX 0x000002CF
#define DDC_TIMEOUT_CNT3_SW_TPI_IVCTX 0x000002D0
#define  DDC_TIMEOUT_CTL_SW_TPI_IVCTX 0x000002D1
#define            PKT_FILTER_4_IVCTX 0x000002D2
#define         DROP_GEN_TYPE_3_IVCTX 0x000002D3
#define         DROP_GEN_TYPE_4_IVCTX 0x000002D4
#define         DROP_GEN_TYPE_5_IVCTX 0x000002D5

//====================AVLINK_REG===============
#define            TX_ZONE_CTL0_IVCTX 0x00000300
#define            TX_ZONE_CTL1_IVCTX 0x00000301
#define            TX_ZONE_CTL2_IVCTX 0x00000302
#define            TX_ZONE_CTL3_IVCTX 0x00000303
#define            TX_ZONE_CTL4_IVCTX 0x00000304
#define            TX_ZONE_CTL5_IVCTX 0x00000305
#define          TX_ZONEVCO_CTL_IVCTX 0x00000306
#define             MHL3CTS_CTL_IVCTX 0x00000307
#define           TX_XBIST_CNTL_IVCTX 0x00000308
#define       TX_XBIST_INST_LOW_IVCTX 0x00000309
#define      TX_XBIST_INST_HIGH_IVCTX 0x0000030A
#define        TX_XBIST_PAT_LOW_IVCTX 0x0000030B
#define       TX_XBIST_PAT_HIGH_IVCTX 0x0000030C
#define       TX_XBIST_CONF_LOW_IVCTX 0x0000030D
#define      TX_XBIST_CONF_HIGH_IVCTX 0x0000030E
#define         TX_XBIST_STATUS_IVCTX 0x0000030F
#define TX_XBIST_PAT_PERIOD_CNT_IVCTX 0x00000310
#define                HDMICTL0_IVCTX 0x00000311
#define                HDMICTL1_IVCTX 0x00000312
#define                HDMICTL2_IVCTX 0x00000313
#define                HDMICTL3_IVCTX 0x00000314
#define                HDMICTL4_IVCTX 0x00000315
#define                HDMICTL5_IVCTX 0x00000316
#define                HDMICTL6_IVCTX 0x00000317
#define                HDMICTL7_IVCTX 0x00000318
#define                  SW_RST_IVCTX 0x00000319
#define        FIFO_10TO20_CTRL_IVCTX 0x0000031A

//====================MHLHDMI_REG===============
#define       MHLHDMITXTOP_INTR_IVCTX 0x00000330
#define      MHLHDMITXTOP_INTR1_IVCTX 0x00000331
#define MHLHDMITXTOP_INTR1_MASK_IVCTX 0x00000332
#define    MHLHDMITXTOP_RDY_RCC_IVCTX 0x00000333
#define       EMSC_ADCTC_LD_SEL_IVCTX 0x00000334

//====================PXL_BIST_REG===============
#define                     BIST_RST_IVCTX 0x00000400
#define            PBIST_H_TOTAL_LOW_IVCTX 0x00000401
#define           PBIST_H_TOTAL_HIGH_IVCTX 0x00000402
#define            PBIST_H_WIDTH_LOW_IVCTX 0x00000403
#define           PBIST_H_WIDTH_HIGH_IVCTX 0x00000404
#define        PBIST_HSYNC_WIDTH_LOW_IVCTX 0x00000405
#define       PBIST_HSYNC_WIDTH_HIGH_IVCTX 0x00000406
#define   PBIST_HSYNC_BACK_PORCH_LOW_IVCTX 0x00000407
#define  PBIST_HSYNC_BACK_PORCH_HIGH_IVCTX 0x00000408
#define  PBIST_HSYNC_FRONT_PORCH_LOW_IVCTX 0x00000409
#define PBIST_HSYNC_FRONT_PORCH_HIGH_IVCTX 0x0000040A
#define            PBIST_V_TOTAL_LOW_IVCTX 0x0000040B
#define           PBIST_V_TOTAL_HIGH_IVCTX 0x0000040C
#define           PBIST_V_HEIGHT_LOW_IVCTX 0x0000040D
#define          PBIST_V_HEIGHT_HIGH_IVCTX 0x0000040E
#define        PBIST_VSYNC_WIDTH_LOW_IVCTX 0x0000040F
#define       PBIST_VSYNC_WIDTH_HIGH_IVCTX 0x00000410
#define   PBIST_VSYNC_BACK_PORCH_LOW_IVCTX 0x00000411
#define  PBIST_VSYNC_BACK_PORCH_HIGH_IVCTX 0x00000412
#define  PBIST_VSYNC_FRONT_PORCH_LOW_IVCTX 0x00000413
#define PBIST_VSYNC_FRONT_PORCH_HIGH_IVCTX 0x00000414
#define             RX_TBIST_CTRL_21_IVCTX 0x00000415
#define        REG_PXL_BIST_BIT_MODE_IVCTX 0x00000416
#define    CONFIG_BIST_10BIT_B_1ST_0_IVCTX 0x00000417
#define    CONFIG_BIST_10BIT_B_1ST_1_IVCTX 0x00000418
#define    CONFIG_BIST_10BIT_B_2ND_0_IVCTX 0x00000419
#define    CONFIG_BIST_10BIT_B_2ND_1_IVCTX 0x0000041A
#define    CONFIG_BIST_10BIT_G_1ST_0_IVCTX 0x0000041B
#define    CONFIG_BIST_10BIT_G_1ST_1_IVCTX 0x0000041C
#define    CONFIG_BIST_10BIT_G_2ND_0_IVCTX 0x0000041D
#define    CONFIG_BIST_10BIT_G_2ND_1_IVCTX 0x0000041E
#define    CONFIG_BIST_10BIT_R_1ST_0_IVCTX 0x0000041F
#define    CONFIG_BIST_10BIT_R_1ST_1_IVCTX 0x00000420
#define    CONFIG_BIST_10BIT_R_2ND_0_IVCTX 0x00000421
#define    CONFIG_BIST_10BIT_R_2ND_1_IVCTX 0x00000422
#define                    STPG_LUM0_IVCTX 0x00000424
#define                    STPG_LUM1_IVCTX 0x00000425
#define                    STPG_LUM2_IVCTX 0x00000426
#define                   STPG_LUM0B_IVCTX 0x00000427
#define                   STPG_LUM1B_IVCTX 0x00000428
#define                   STPG_LUM2B_IVCTX 0x00000429
#define                REG_V5_H4TO13_IVCTX 0x0000042A
#define                    REG_V6_H2_IVCTX 0x0000042B
#define                    REG_V6_H3_IVCTX 0x0000042C
#define                    REG_V6_H4_IVCTX 0x0000042D
#define                    REG_V6_H5_IVCTX 0x0000042E
#define                    REG_V6_H6_IVCTX 0x0000042F
#define                    REG_V6_H7_IVCTX 0x00000430
#define                    REG_V6_H8_IVCTX 0x00000431
#define                    REG_V6_H9_IVCTX 0x00000432
#define                   REG_V6_H10_IVCTX 0x00000433
#define                   REG_V6_H11_IVCTX 0x00000434
#define                   REG_V6_H12_IVCTX 0x00000435
#define                   REG_V6_H13_IVCTX 0x00000436
#define                   REG_V6_H14_IVCTX 0x00000437
#define                   REG_V6_H15_IVCTX 0x00000438
#define                   REG_V6_H16_IVCTX 0x00000439
#define               REG_V7_CONST61_IVCTX 0x0000043A
#define               REG_V7_CONST50_IVCTX 0x0000043B
#define              REG_V7_CONST102_IVCTX 0x0000043C
#define              REG_V9_CONST204_IVCTX 0x0000043D
#define              REG_V9_CONST205_IVCTX 0x0000043E
#define              REG_V9_CONST209_IVCTX 0x0000043F
#define              REG_V9_CONST229_IVCTX 0x00000440
#define              REG_V9_CONST231_IVCTX 0x00000441
#define                REG_V10_H3TO4_IVCTX 0x00000442
#define                REG_V10_H5TO6_IVCTX 0x00000443
#define                REG_V10_H7TO8_IVCTX 0x00000444
#define               REG_V10_H9TO10_IVCTX 0x00000445
#define              REG_V10_H11TO12_IVCTX 0x00000446
#define              REG_V10_H13TO14_IVCTX 0x00000447
#define                    BIST_CTRL_IVCTX 0x00000448
#define               BIST_DURATION0_IVCTX 0x00000449
#define               BIST_DURATION1_IVCTX 0x0000044A
#define               BIST_DURATION2_IVCTX 0x0000044B
#define                BIST_TEST_SEL_IVCTX 0x0000044C
#define              BIST_VIDEO_MODE_IVCTX 0x0000044D
#define            BIST_8BIT_PATTERN_IVCTX 0x0000044E
#define                  BIST_STATUS_IVCTX 0x00000451
#define                  BIST_RESULT_IVCTX 0x00000452
#define             BIST_P_ERR_CNT_0_IVCTX 0x00000453
#define             BIST_P_ERR_CNT_1_IVCTX 0x00000454
#define             BIST_R_ERR_CNT_0_IVCTX 0x00000455
#define             BIST_R_ERR_CNT_1_IVCTX 0x00000456
#define             BIST_G_ERR_CNT_0_IVCTX 0x00000457
#define             BIST_G_ERR_CNT_1_IVCTX 0x00000458
#define             BIST_B_ERR_CNT_0_IVCTX 0x00000459
#define             BIST_B_ERR_CNT_1_IVCTX 0x0000045A
#define            BIST_CNTL_ERR_CNT_IVCTX 0x0000045B
#define                   STPG_SIZE1_IVCTX 0x0000045C
#define                   STPG_SIZE2_IVCTX 0x0000045D
#define                   STPG_SIZE3_IVCTX 0x0000045E
#define                   BIST_CTRL2_IVCTX 0x00000465
#define             BIST_TIMING_CTRL_IVCTX 0x00000466
#define      BIST_EXTD_CNT_ERR_CNT_0_IVCTX 0x00000467
#define      BIST_EXTD_CNT_ERR_CNT_1_IVCTX 0x00000468
#define         BIST_EXTD_COLOR_MODE_IVCTX 0x00000469
#define                STPG_EXTD_RED_IVCTX 0x0000046A
#define               STPG_EXTD_BLUE_IVCTX 0x0000046B
#define              STPG_EXTD_GREEN_IVCTX 0x0000046C
#define  BIST_STPG_EXTD_BUS_CNT_MODE_IVCTX 0x0000046D
#define         STPG_EXTD_CNT_STATIC_IVCTX 0x0000046E
#define  BIST_PGEN_EXTD_BUS_CNT_MODE_IVCTX 0x0000046F
#define         PGEN_EXTD_CNT_STATIC_IVCTX 0x00000470
#define                  Y_WHITE_LOW_IVCTX 0x00000471
#define                 Y_WHITE_HIGH_IVCTX 0x00000472
#define                 CB_WHITE_LOW_IVCTX 0x00000473
#define                CB_WHITE_HIGH_IVCTX 0x00000474
#define                 CR_WHITE_LOW_IVCTX 0x00000475
#define                CR_WHITE_HIGH_IVCTX 0x00000476
#define                 Y_YELLOW_LOW_IVCTX 0x00000477
#define                Y_YELLOW_HIGH_IVCTX 0x00000478
#define                CB_YELLOW_LOW_IVCTX 0x00000479
#define               CB_YELLOW_HIGH_IVCTX 0x0000047A
#define                CR_YELLOW_LOW_IVCTX 0x0000047B
#define               CR_YELLOW_HIGH_IVCTX 0x0000047C
#define                   Y_CYAN_LOW_IVCTX 0x0000047D
#define                  Y_CYAN_HIGH_IVCTX 0x0000047E
#define                  CB_CYAN_LOW_IVCTX 0x0000047F
#define                 CB_CYAN_HIGH_IVCTX 0x00000480
#define                  CR_CYAN_LOW_IVCTX 0x00000481
#define                 CR_CYAN_HIGH_IVCTX 0x00000482
#define                  Y_GREEN_LOW_IVCTX 0x00000483
#define                 Y_GREEN_HIGH_IVCTX 0x00000484
#define                 CB_GREEN_LOW_IVCTX 0x00000485
#define                CB_GREEN_HIGH_IVCTX 0x00000486
#define                 CR_GREEN_LOW_IVCTX 0x00000487
#define                CR_GREEN_HIGH_IVCTX 0x00000488
#define                Y_MAGENTA_LOW_IVCTX 0x00000489
#define               Y_MAGENTA_HIGH_IVCTX 0x0000048A
#define               CB_MAGENTA_LOW_IVCTX 0x0000048B
#define              CB_MAGENTA_HIGH_IVCTX 0x0000048C
#define               CR_MAGENTA_LOW_IVCTX 0x0000048D
#define              CR_MAGENTA_HIGH_IVCTX 0x0000048E
#define                    Y_RED_LOW_IVCTX 0x0000048F
#define                   Y_RED_HIGH_IVCTX 0x00000490
#define                   CB_RED_LOW_IVCTX 0x00000491
#define                  CB_RED_HIGH_IVCTX 0x00000492
#define                   CR_RED_LOW_IVCTX 0x00000493
#define                  CR_RED_HIGH_IVCTX 0x00000494
#define                   Y_BLUE_LOW_IVCTX 0x00000495
#define                  Y_BLUE_HIGH_IVCTX 0x00000496
#define                  CB_BLUE_LOW_IVCTX 0x00000497
#define                 CB_BLUE_HIGH_IVCTX 0x00000498
#define                  CR_BLUE_LOW_IVCTX 0x00000499
#define                 CR_BLUE_HIGH_IVCTX 0x0000049A
#define                  Y_BLACK_LOW_IVCTX 0x0000049B
#define                 Y_BLACK_HIGH_IVCTX 0x0000049C
#define                 CB_BLACK_LOW_IVCTX 0x0000049D
#define                CB_BLACK_HIGH_IVCTX 0x0000049E
#define                 CR_BLACK_LOW_IVCTX 0x0000049F
#define                CR_BLACK_HIGH_IVCTX 0x000004A0

//====================HW_TPI_REG===============
#define                       TPI_MISC_IVCTX 0x00000607
#define                        TPI_PRD_IVCTX 0x00000608
#define                      TPI_INPUT_IVCTX 0x00000609
#define                     TPI_OUTPUT_IVCTX 0x0000060A
#define                  TPI_AVI_CHSUM_IVCTX 0x0000060C
#define                  TPI_AVI_BYTE1_IVCTX 0x0000060D
#define                  TPI_AVI_BYTE2_IVCTX 0x0000060E
#define                  TPI_AVI_BYTE3_IVCTX 0x0000060F
#define                  TPI_AVI_BYTE4_IVCTX 0x00000610
#define                  TPI_AVI_BYTE5_IVCTX 0x00000611
#define                  TPI_AVI_BYTE6_IVCTX 0x00000612
#define                  TPI_AVI_BYTE7_IVCTX 0x00000613
#define                  TPI_AVI_BYTE8_IVCTX 0x00000614
#define                  TPI_AVI_BYTE9_IVCTX 0x00000615
#define                 TPI_AVI_BYTE10_IVCTX 0x00000616
#define                 TPI_AVI_BYTE11_IVCTX 0x00000617
#define                 TPI_AVI_BYTE12_IVCTX 0x00000618
#define                 TPI_AVI_BYTE13_IVCTX 0x00000619
#define                         TPI_SC_IVCTX 0x0000061A
#define                     TPI_DEV_ID_IVCTX 0x0000061B
#define                 TPI_COPP_DATA1_IVCTX 0x00000629
#define                 TPI_COPP_DATA2_IVCTX 0x0000062A
#define                  TPI_WR_BKSV_1_IVCTX 0x0000062B
#define                  TPI_WR_BKSV_2_IVCTX 0x0000062C
#define                  TPI_WR_BKSV_3_IVCTX 0x0000062D
#define                  TPI_WR_BKSV_4_IVCTX 0x0000062E
#define                  TPI_WR_BKSV_5_IVCTX 0x0000062F
#define                   TPI_HDCP_REV_IVCTX 0x00000630
#define                      TPI_KSV_V_IVCTX 0x00000631
#define                  TPI_VVALUE_B0_IVCTX 0x00000632
#define                  TPI_VVALUE_B1_IVCTX 0x00000633
#define                  TPI_VVALUE_B2_IVCTX 0x00000634
#define                  TPI_VVALUE_B3_IVCTX 0x00000635
#define                     TPI_AKSV_1_IVCTX 0x00000636
#define                     TPI_AKSV_2_IVCTX 0x00000637
#define                     TPI_AKSV_3_IVCTX 0x00000638
#define                     TPI_AKSV_4_IVCTX 0x00000639
#define                     TPI_AKSV_5_IVCTX 0x0000063A
#define            TPI_HPD_RSEN_SW_TPI_IVCTX 0x0000063B
#define                    TPI_INTR_EN_IVCTX 0x0000063C
#define                   TPI_INTR_ST0_IVCTX 0x0000063D
#define              TPI_KSV_FIFO_STAT_IVCTX 0x00000641
#define              TPI_KSV_FIFO_FORW_IVCTX 0x00000642
#define                   TPI_DS_BCAPS_IVCTX 0x00000644
#define                   TPI_BSTATUS1_IVCTX 0x00000645
#define                   TPI_BSTATUS2_IVCTX 0x00000646
#define                  TPI_VID_MUTE0_IVCTX 0x0000064B
#define                   RSVD1_HW_TPI_IVCTX 0x0000064C
#define                  TPI_VID_MUTE1_IVCTX 0x0000064D
#define                   RSVD2_HW_TPI_IVCTX 0x0000064E
#define                  TPI_VID_MUTE2_IVCTX 0x0000064F
#define                   RSVD4_HW_TPI_IVCTX 0x00000660
#define                    TPI_HW_DBG1_IVCTX 0x00000679
#define                    TPI_HW_DBG2_IVCTX 0x0000067A
#define                    TPI_HW_DBG3_IVCTX 0x0000067B
#define                    TPI_HW_DBG4_IVCTX 0x0000067C
#define                    TPI_HW_DBG5_IVCTX 0x0000067D
#define                    TPI_HW_DBG6_IVCTX 0x0000067E
#define                    TPI_HW_DBG7_IVCTX 0x0000067F
#define                    TPI_HW_DBG8_IVCTX 0x00000680
#define                    TPI_HW_DBG9_IVCTX 0x00000681
#define                   TPI_HW_DBG10_IVCTX 0x00000682
#define                   TPI_HW_DBG11_IVCTX 0x00000683
#define                   TPI_HW_DBG12_IVCTX 0x00000684
#define                   TPI_HW_DBG13_IVCTX 0x00000685
#define                   TPI_HW_DBG14_IVCTX 0x00000686
#define                   TPI_HW_DBG15_IVCTX 0x00000687
#define                   TPI_HW_DBG16_IVCTX 0x00000688
#define                    TPI_HW_OPT0_IVCTX 0x000006B8
#define                    TPI_HW_OPT1_IVCTX 0x000006B9
#define                    TPI_HW_OPT2_IVCTX 0x000006BA
#define                    TPI_HW_OPT3_IVCTX 0x000006BB
#define                   RSVD7_HW_TPI_IVCTX 0x000006BC
#define                   RSVD8_HW_TPI_IVCTX 0x000006BD
#define                   RSVD9_HW_TPI_IVCTX 0x000006BE
#define                  TPI_INFO_FSEL_IVCTX 0x000006BF
#define                    TPI_INFO_B0_IVCTX 0x000006C0
#define                    TPI_INFO_B1_IVCTX 0x000006C1
#define                    TPI_INFO_B2_IVCTX 0x000006C2
#define                    TPI_INFO_B3_IVCTX 0x000006C3
#define                    TPI_INFO_B4_IVCTX 0x000006C4
#define                    TPI_INFO_B5_IVCTX 0x000006C5
#define                    TPI_INFO_B6_IVCTX 0x000006C6
#define                    TPI_INFO_B7_IVCTX 0x000006C7
#define                    TPI_INFO_B8_IVCTX 0x000006C8
#define                    TPI_INFO_B9_IVCTX 0x000006C9
#define                   TPI_INFO_B10_IVCTX 0x000006CA
#define                   TPI_INFO_B11_IVCTX 0x000006CB
#define                   TPI_INFO_B12_IVCTX 0x000006CC
#define                   TPI_INFO_B13_IVCTX 0x000006CD
#define                   TPI_INFO_B14_IVCTX 0x000006CE
#define                   TPI_INFO_B15_IVCTX 0x000006CF
#define                   TPI_INFO_B16_IVCTX 0x000006D0
#define                   TPI_INFO_B17_IVCTX 0x000006D1
#define                   TPI_INFO_B18_IVCTX 0x000006D2
#define                   TPI_INFO_B19_IVCTX 0x000006D3
#define                   TPI_INFO_B20_IVCTX 0x000006D4
#define                   TPI_INFO_B21_IVCTX 0x000006D5
#define                   TPI_INFO_B22_IVCTX 0x000006D6
#define                   TPI_INFO_B23_IVCTX 0x000006D7
#define                   TPI_INFO_B24_IVCTX 0x000006D8
#define                   TPI_INFO_B25_IVCTX 0x000006D9
#define                   TPI_INFO_B26_IVCTX 0x000006DA
#define                   TPI_INFO_B27_IVCTX 0x000006DB
#define                   TPI_INFO_B28_IVCTX 0x000006DC
#define                   TPI_INFO_B29_IVCTX 0x000006DD
#define                   TPI_INFO_B30_IVCTX 0x000006DE
#define                    TPI_INFO_EN_IVCTX 0x000006DF
#define                 VSIF_COMP_CNTL_IVCTX 0x000006E0
#define                       GCP_CNTL_IVCTX 0x000006E1
#define                    GCP_GEN_CNT_IVCTX 0x000006E2
#define                   GCP_AUTO_GEN_IVCTX 0x000006E3
#define                GCP_AUTO_GEN_EN_IVCTX 0x000006E4
#define                   GCP_CUR_STAT_IVCTX 0x000006E5
#define            AUTO_BLANK_VID_CTRL_IVCTX 0x000006E6
#define          R_CR_BLANK_DATA_REG_0_IVCTX 0x000006E7
#define          R_CR_BLANK_DATA_REG_1_IVCTX 0x000006E8
#define           G_Y_BLANK_DATA_REG_0_IVCTX 0x000006E9
#define           G_Y_BLANK_DATA_REG_1_IVCTX 0x000006EA
#define          B_CB_BLANK_DATA_REG_0_IVCTX 0x000006EB
#define          B_CB_BLANK_DATA_REG_1_IVCTX 0x000006EC
#define                      VTEM_CTRL_IVCTX 0x000006ED
#define HDMI_PIPELINE_DI_VSYNCOFFSET_0_IVCTX 0x000006EE
#define HDMI_PIPELINE_DI_VSYNCOFFSET_1_IVCTX 0x000006EF
#define   HDMI_PIPELINE_DI_VSYNC_DELTA_IVCTX 0x000006F0
#define    HDMI_PIPELINE_DI_VSYNC_CTRL_IVCTX 0x000006F1
#define             HDMI_PIPELINE_GP_0_IVCTX 0x000006F2
#define             HDMI_PIPELINE_GP_1_IVCTX 0x000006F3
#define             HDMI_PIPELINE_GP_2_IVCTX 0x000006F4
#define             HDMI_PIPELINE_GP_3_IVCTX 0x000006F5
#define                  RSVD10_HW_TPI_IVCTX 0x000006F7
#define              TPI_DDC_MASTER_EN_IVCTX 0x000006F8
#define          TPI_VSIF_VX1CTL_HW_EN_IVCTX 0x000006F9
#define              TPI_VX1CTL_M0_MSB_IVCTX 0x000006FA
#define              TPI_VX1CTL_M0_LSB_IVCTX 0x000006FB
#define              TPI_VX1CTL_M1_MSB_IVCTX 0x000006FC
#define              TPI_VX1CTL_M1_LSB_IVCTX 0x000006FD
#define                  RSVD11_HW_TPI_IVCTX 0x000006FF

//====================HUNTER_PHY_REG===============
#define      HT_TOP_CTL_PHY_IVCTX 0x00000700
#define      HT_DP_CTL2_PHY_IVCTX 0x00000703
#define      HT_DP_CTL3_PHY_IVCTX 0x00000704
#define      HT_DP_CTL4_PHY_IVCTX 0x00000705
#define     HT_DRV_CTL0_PHY_IVCTX 0x00000706
#define     HT_PLL_CTL0_PHY_IVCTX 0x0000072A
#define     HT_PLL_CTL7_PHY_IVCTX 0x00000731
#define     HT_DIG_CTL0_PHY_IVCTX 0x00000748
#define     HT_DIG_CTL1_PHY_IVCTX 0x00000749
#define     HT_DIG_CTL2_PHY_IVCTX 0x0000074A
#define     HT_DIG_CTL3_PHY_IVCTX 0x0000074B
#define     HT_DIG_CTL4_PHY_IVCTX 0x0000074C
#define     HT_DIG_CTL5_PHY_IVCTX 0x0000074D
#define     HT_DIG_CTL6_PHY_IVCTX 0x0000074E
#define     HT_DIG_CTL7_PHY_IVCTX 0x0000074F
#define     HT_DIG_CTL8_PHY_IVCTX 0x00000750
#define     HT_DIG_CTL9_PHY_IVCTX 0x00000751
#define    HT_DIG_CTL10_PHY_IVCTX 0x00000752
#define    HT_DIG_CTL11_PHY_IVCTX 0x00000753
#define    HT_DIG_CTL12_PHY_IVCTX 0x00000754
#define    HT_DIG_CTL13_PHY_IVCTX 0x00000755
#define    HT_DIG_CTL14_PHY_IVCTX 0x00000756
#define    HT_DIG_CTL15_PHY_IVCTX 0x00000757
#define    HT_DIG_CTL16_PHY_IVCTX 0x00000758
#define    HT_DIG_CTL17_PHY_IVCTX 0x00000759
#define    HT_DIG_CTL18_PHY_IVCTX 0x0000075A
#define    HT_DIG_CTL19_PHY_IVCTX 0x0000075B
#define    HT_DIG_CTL20_PHY_IVCTX 0x0000075C
#define    HT_DIG_CTL21_PHY_IVCTX 0x0000075D
#define    HT_DIG_CTL22_PHY_IVCTX 0x0000075E
#define    HT_DIG_CTL23_PHY_IVCTX 0x0000075F
#define    HT_DIG_CTL24_PHY_IVCTX 0x00000760
#define HUNTER_ANA_ST_0_PHY_IVCTX 0x00000761
#define HT_DIG_PAT100_0_PHY_IVCTX 0x00000762
#define HT_DIG_PAT100_1_PHY_IVCTX 0x00000763
#define HT_DIG_PAT100_2_PHY_IVCTX 0x00000764
#define HT_DIG_PAT100_3_PHY_IVCTX 0x00000765
#define HT_DIG_PAT100_4_PHY_IVCTX 0x00000766
#define HT_DIG_PAT100_5_PHY_IVCTX 0x00000767
#define HT_DIG_PAT100_6_PHY_IVCTX 0x00000768
#define HT_DIG_PAT100_7_PHY_IVCTX 0x00000769
#define HT_DIG_PAT100_8_PHY_IVCTX 0x0000076A
#define HT_DIG_PAT100_9_PHY_IVCTX 0x0000076B
#define HT_DIG_PAT100_A_PHY_IVCTX 0x0000076C
#define HT_DIG_PAT100_B_PHY_IVCTX 0x0000076D
#define HT_DIG_PAT100_C_PHY_IVCTX 0x0000076E
#define  HT_DIG_PAT_CTL_PHY_IVCTX 0x0000076F
#define     HT_DBG_CTL0_PHY_IVCTX 0x00000770
#define     HT_DBG_CTL1_PHY_IVCTX 0x00000771
#define     HT_DBG_CTL2_PHY_IVCTX 0x00000772
#define     HT_DBG_CTL3_PHY_IVCTX 0x00000773
#define     HT_DBG_CTL4_PHY_IVCTX 0x00000774
#define       HT_LTP_ST_PHY_IVCTX 0x00000777

//====================HDCP2XCORE_REG===============
#define         CP2TX_CTRL_0_IVCTX 0x00000800
#define         CP2TX_CTRL_1_IVCTX 0x00000801
#define         CP2TX_CTRL_2_IVCTX 0x00000802
#define          CP2TX_INTR0_IVCTX 0x00000803
#define          CP2TX_INTR1_IVCTX 0x00000804
#define          CP2TX_INTR2_IVCTX 0x00000805
#define          CP2TX_INTR3_IVCTX 0x00000806
#define     CP2TX_INTR0_MASK_IVCTX 0x00000807
#define     CP2TX_INTR1_MASK_IVCTX 0x00000808
#define     CP2TX_INTR2_MASK_IVCTX 0x00000809
#define     CP2TX_INTR3_MASK_IVCTX 0x0000080A
#define     CP2TX_INTRSTATUS_IVCTX 0x0000080B
#define      CP2TX_AUTH_STAT_IVCTX 0x0000080C
#define          CP2TX_STATE_IVCTX 0x0000080D
#define     CP2TX_GEN_STATUS_IVCTX 0x0000080E
#define      CP2TX_ROSC_BIST_IVCTX 0x0000080F
#define            CP2TX_TP0_IVCTX 0x00000810
#define            CP2TX_TP1_IVCTX 0x00000811
#define            CP2TX_TP2_IVCTX 0x00000812
#define            CP2TX_TP3_IVCTX 0x00000813
#define            CP2TX_TP4_IVCTX 0x00000814
#define            CP2TX_TP5_IVCTX 0x00000815
#define            CP2TX_TP6_IVCTX 0x00000816
#define            CP2TX_TP7_IVCTX 0x00000817
#define            CP2TX_TP8_IVCTX 0x00000818
#define            CP2TX_TP9_IVCTX 0x00000819
#define           CP2TX_TP10_IVCTX 0x0000081A
#define           CP2TX_TP11_IVCTX 0x0000081B
#define           CP2TX_TP12_IVCTX 0x0000081C
#define           CP2TX_TP13_IVCTX 0x0000081D
#define           CP2TX_TP14_IVCTX 0x0000081E
#define           CP2TX_TP15_IVCTX 0x0000081F
#define         CP2TX_GP_IN0_IVCTX 0x00000820
#define         CP2TX_GP_IN1_IVCTX 0x00000821
#define         CP2TX_GP_IN2_IVCTX 0x00000822
#define         CP2TX_GP_IN3_IVCTX 0x00000823
#define         CP2TX_GP_IN4_IVCTX 0x00000824
#define         CP2TX_GP_IN5_IVCTX 0x00000825
#define          CP2TX_GPCTL_IVCTX 0x00000826
#define        CP2TX_GP_OUT0_IVCTX 0x00000827
#define        CP2TX_GP_OUT1_IVCTX 0x00000828
#define        CP2TX_GP_OUT2_IVCTX 0x00000829
#define        CP2TX_GP_OUT3_IVCTX 0x0000082A
#define        CP2TX_GP_OUT4_IVCTX 0x0000082B
#define        CP2TX_GP_OUT5_IVCTX 0x0000082C
#define   CP2TX_RX_ID_CORE_0_IVCTX 0x0000082D
#define   CP2TX_RX_ID_CORE_1_IVCTX 0x0000082E
#define   CP2TX_RX_ID_CORE_2_IVCTX 0x0000082F
#define   CP2TX_RX_ID_CORE_3_IVCTX 0x00000830
#define   CP2TX_RX_ID_CORE_4_IVCTX 0x00000831
#define     CP2TX_RPT_DETAIL_IVCTX 0x00000832
#define     CP2TX_RPT_SMNG_K_IVCTX 0x00000833
#define      CP2TX_RPT_DEPTH_IVCTX 0x00000834
#define     CP2TX_RPT_DEVCNT_IVCTX 0x00000835
#define    CP2TX_SEQ_NUM_V_0_IVCTX 0x00000836
#define    CP2TX_SEQ_NUM_V_1_IVCTX 0x00000837
#define    CP2TX_SEQ_NUM_V_2_IVCTX 0x00000838
#define    CP2TX_SEQ_NUM_M_0_IVCTX 0x00000839
#define    CP2TX_SEQ_NUM_M_1_IVCTX 0x0000083A
#define    CP2TX_SEQ_NUM_M_2_IVCTX 0x0000083B
#define   CP2TX_IPT_CTR_7TO0_IVCTX 0x0000083C
#define  CP2TX_IPT_CTR_15TO8_IVCTX 0x0000083D
#define         CP2TX_AESCTL_IVCTX 0x0000083E
#define      CP2TX_TX_CTRL_0_IVCTX 0x00000870
#define      CP2TX_TX_STATUS_IVCTX 0x00000871
#define CP2TX_TX_RPT_SMNG_IN_IVCTX 0x00000872
#define  CP2TX_RPT_RCVID_OUT_IVCTX 0x00000873
#define    CP2TX_CIPHER_CTL2_IVCTX 0x00000874
#define            CP2TX_GP0_IVCTX 0x00000875
#define            CP2TX_GP1_IVCTX 0x00000876

//====================HDCP2XWRAPPER_REG===============
#define             HDCP2X_DEBUG_CTRL0_IVCTX 0x000008A0
#define             HDCP2X_DEBUG_CTRL1_IVCTX 0x000008A1
#define             HDCP2X_DEBUG_CTRL2_IVCTX 0x000008A2
#define             HDCP2X_DEBUG_CTRL3_IVCTX 0x000008A3
#define             HDCP2X_DEBUG_CTRL4_IVCTX 0x000008A4
#define             HDCP2X_DEBUG_STAT0_IVCTX 0x000008A5
#define             HDCP2X_DEBUG_STAT1_IVCTX 0x000008A6
#define             HDCP2X_DEBUG_STAT2_IVCTX 0x000008A7
#define             HDCP2X_DEBUG_STAT3_IVCTX 0x000008A8
#define             HDCP2X_DEBUG_STAT4_IVCTX 0x000008A9
#define             HDCP2X_DEBUG_STAT5_IVCTX 0x000008AA
#define             HDCP2X_DEBUG_STAT6_IVCTX 0x000008AB
#define             HDCP2X_DEBUG_STAT7_IVCTX 0x000008AC
#define             HDCP2X_DEBUG_STAT8_IVCTX 0x000008AD
#define             HDCP2X_DEBUG_STAT9_IVCTX 0x000008AE
#define            HDCP2X_DEBUG_STAT10_IVCTX 0x000008AF
#define            HDCP2X_DEBUG_STAT11_IVCTX 0x000008B0
#define            HDCP2X_DEBUG_STAT12_IVCTX 0x000008B1
#define            HDCP2X_DEBUG_STAT13_IVCTX 0x000008B2
#define            HDCP2X_DEBUG_STAT14_IVCTX 0x000008B3
#define            HDCP2X_DEBUG_STAT15_IVCTX 0x000008B4
#define                 HDCP2X_TX_SRST_IVCTX 0x000008B5
#define                 HDCP2X_POLL_CS_IVCTX 0x000008B6
#define      HDCP2X_CUPD_START_ADDR_LO_IVCTX 0x000008B7
#define      HDCP2X_CUPD_START_ADDR_HI_IVCTX 0x000008B8
#define HDCP2X_CUPD_SIGN_START_ADDR_LO_IVCTX 0x000008B9
#define HDCP2X_CUPD_SIGN_START_ADDR_HI_IVCTX 0x000008BA
#define   HDCP2X_PRAM_SIGN_END_ADDR_LO_IVCTX 0x000008BB
#define   HDCP2X_CUPD_SIGN_END_ADDR_HI_IVCTX 0x000008BC
#define                   HDCP2X_CTL_0_IVCTX 0x000008BD
#define                   HDCP2X_CTL_1_IVCTX 0x000008BE
#define                   HDCP2X_CTL_2_IVCTX 0x000008BF
#define            HDCP2X_CUPD_SIZE_LO_IVCTX 0x000008C0
#define            HDCP2X_CUPD_SIZE_HI_IVCTX 0x000008C1
#define                 HDCP2X_GEN_STA_IVCTX 0x000008C2
#define               HDCP2X_POLL_VAL0_IVCTX 0x000008C3
#define               HDCP2X_POLL_VAL1_IVCTX 0x000008C4
#define                HDCP2X_DDCM_STS_IVCTX 0x000008C5
#define               HDCP2X_ROSC_BIST_IVCTX 0x000008C6
#define                HDCP2_PRAM_DATA_IVCTX 0x000008CF
#define            HDCP2X_DEBUG_STAT16_IVCTX 0x000008D0

//====================HDMI2_REG===============
#define            SCRCTL_IVCTX 0x00000900
#define         HDMI2CTL0_IVCTX 0x00000901
#define         HDMI2CTL1_IVCTX 0x00000902
#define        TXC_DATA0L_IVCTX 0x00000903
#define        TXC_DATA0H_IVCTX 0x00000904
#define        TXC_DATA1L_IVCTX 0x00000905
#define        TXC_DATA1H_IVCTX 0x00000906
#define        TXC_DATA2L_IVCTX 0x00000907
#define        TXC_DATA2H_IVCTX 0x00000908
#define      TXC_DATA_DIV_IVCTX 0x00000909
#define       RSVD0_HDMI2_IVCTX 0x0000090A
#define          SCDC_CTL_IVCTX 0x00000920
#define   SCDC_ACT_STATUS_IVCTX 0x00000921
#define        SCDC_STATE_IVCTX 0x00000922
#define     SCDC_UP_FLAG0_IVCTX 0x00000923
#define     SCDC_UP_FLAG1_IVCTX 0x00000924
#define        SCDC_INTR0_IVCTX 0x00000925
#define   SCDC_INTR0_MASK_IVCTX 0x00000926
#define  SCDC_POLL_TIMER0_IVCTX 0x00000927
#define  SCDC_POLL_TIMER1_IVCTX 0x00000928
#define  SCDC_POLL_TIMER2_IVCTX 0x00000929
#define         SCDC_SRST_IVCTX 0x0000092A
#define    SCDC_FRM_LIMIT_IVCTX 0x0000092B
#define    RREQ_FRM_LIMIT_IVCTX 0x0000092C
#define SCDC_SRC_TEST_CFG_IVCTX 0x0000092D
#define    SCDC_STS_FLAG0_IVCTX 0x0000092E
#define    SCDC_STS_FLAG1_IVCTX 0x0000092F
#define    SCDC_STS_FLAG2_IVCTX 0x00000930
#define    FRL_LTP_OVR_EN_IVCTX 0x00000931
#define  FRL_LTP_OVR_VAL0_IVCTX 0x00000932
#define  FRL_LTP_OVR_VAL1_IVCTX 0x00000933
#define       RSVD1_HDMI2_IVCTX 0x00000934

//====================H21TX_SB_REG=============
#define                   H21TXSB_CTRL_IVCTX 0x00000940
#define                H21TXSB_PKT_PRD_IVCTX 0x00000941
#define                 H21TXSB_MAXPKT_IVCTX 0x00000942
#define                  H21TXSB_SRPRD_IVCTX 0x00000943
#define                    H21TXSB_KSR_IVCTX 0x00000944
#define                   H21TXSB_KSSB_IVCTX 0x00000945
#define                    H21TXSB_CPR_IVCTX 0x00000946
#define                    H21TXSB_SPM_IVCTX 0x00000947
#define               H21TXSB_PATT_SEL_IVCTX 0x00000948
#define             H21TXSB_ERR_INS_EN_IVCTX 0x00000949
#define H21TXSB_ERR_INS_BYTE_WISE_EN_0_IVCTX 0x0000094A
#define H21TXSB_ERR_INS_BYTE_WISE_EN_1_IVCTX 0x0000094B
#define H21TXSB_ERR_INS_BYTE_WISE_EN_2_IVCTX 0x0000094C
#define H21TXSB_ERR_INS_BYTE_WISE_EN_3_IVCTX 0x0000094D
#define H21TXSB_ERR_INS_BYTE_WISE_EN_4_IVCTX 0x0000094E
#define H21TXSB_ERR_INS_BYTE_WISE_EN_5_IVCTX 0x0000094F
#define H21TXSB_ERR_INS_BYTE_WISE_EN_6_IVCTX 0x00000950
#define H21TXSB_ERR_INS_BYTE_WISE_EN_7_IVCTX 0x00000951
#define           H21TXSB_BLOCK0_BYTE0_IVCTX 0x00000952
#define           H21TXSB_BLOCK0_BYTE1_IVCTX 0x00000953
#define           H21TXSB_BLOCK0_BYTE2_IVCTX 0x00000954
#define           H21TXSB_BLOCK1_BYTE0_IVCTX 0x00000955
#define            H21TXSB_BOCK1_BYTE1_IVCTX 0x00000956
#define            H21TXSB_BOCK1_BYTE2_IVCTX 0x00000957
#define           H21TXSB_BLOCK2_BYTE0_IVCTX 0x00000958
#define           H21TXSB_BLOCK2_BYTE1_IVCTX 0x00000959
#define           H21TXSB_BLOCK2_BYTE2_IVCTX 0x0000095A
#define           H21TXSB_BLOCK3_BYTE0_IVCTX 0x0000095B
#define           H21TXSB_BLOCK3_BYTE1_IVCTX 0x0000095C
#define           H21TXSB_BLOCK3_BYTE2_IVCTX 0x0000095D
#define  H21TXSB_BLOCK0_BYTE0_CTRL_VAL_IVCTX 0x0000095E
#define  H21TXSB_BLOCK0_BYTE1_CTRL_VAL_IVCTX 0x0000095F
#define  H21TXSB_BLOCK0_BYTE2_CTRL_VAL_IVCTX 0x00000960
#define  H21TXSB_BLOCK1_BYTE0_CTRL_VAL_IVCTX 0x00000961
#define  H21TXSB_BLOCK1_BYTE1_CTRL_VAL_IVCTX 0x00000962
#define  H21TXSB_BLOCK1_BYTE2_CTRL_VAL_IVCTX 0x00000963
#define  H21TXSB_BLOCK2_BYTE0_CTRL_VAL_IVCTX 0x00000964
#define  H21TXSB_BLOCK2_BYTE1_CTRL_VAL_IVCTX 0x00000965
#define  H21TXSB_BLOCK2_BYTE2_CTRL_VAL_IVCTX 0x00000966
#define  H21TXSB_BLOCK3_BYTE0_CTRL_VAL_IVCTX 0x00000967
#define  H21TXSB_BLOCK3_BYTE1_CTRL_VAL_IVCTX 0x00000968
#define  H21TXSB_BLOCK3_BYTE2_CTRL_VAL_IVCTX 0x00000969
#define                H21TXSB_INTRSTS_IVCTX 0x0000096A
#define         H21TXSB_D_HDR_INTRMASK_IVCTX 0x0000096B
#define   H21TXSB_KEEPOUT_PERIOD_END_0_IVCTX 0x0000096C
#define   H21TXSB_KEEPOUT_PERIOD_END_1_IVCTX 0x0000096D
#define H21TXSB_KEEPOUT_PERIOD_START_0_IVCTX 0x0000096E
#define H21TXSB_KEEPOUT_PERIOD_START_1_IVCTX 0x0000096F
#define                 H21TXSB_CTRL_1_IVCTX 0x00000970
#define                H21TXSB_SPARE_0_IVCTX 0x00000971
#define                H21TXSB_SPARE_1_IVCTX 0x00000972
#define                H21TXSB_SPARE_2_IVCTX 0x00000973
#define                H21TXSB_SPARE_3_IVCTX 0x00000974
#define                H21TXSB_SPARE_4_IVCTX 0x00000975
#define                H21TXSB_SPARE_5_IVCTX 0x00000976
#define                H21TXSB_SPARE_6_IVCTX 0x00000977
#define                H21TXSB_SPARE_7_IVCTX 0x00000978
#define                H21TXSB_SPARE_8_IVCTX 0x00000979
#define                H21TXSB_SPARE_9_IVCTX 0x0000097A

//==============H21TX_SB_TOP_REG===========
#define                 H21TX_SB_TOP0_IVCTX 0x00000980
#define                 H21TX_SB_TOP1_IVCTX 0x00000981
#define    H21TX_SB_TOP_INS_DISP_CTRL_IVCTX 0x00000982
#define H21TX_SB_TOP_INS_DISP_10B_CNT_IVCTX 0x00000983
#define H21TX_SB_TOP_INS_DISP_08B_CNT_IVCTX 0x00000984
#define  H21TX_SB_TOP_INS_DISP_CTRL_1_IVCTX 0x00000985

//====================AIP_REG===============
#define            RSVD0_AIP_IVCTX 0x00000A00
#define             ACR_CTRL_IVCTX 0x00000A01
#define            FREQ_SVAL_IVCTX 0x00000A02
#define              N_SVAL1_IVCTX 0x00000A03
#define              N_SVAL2_IVCTX 0x00000A04
#define              N_SVAL3_IVCTX 0x00000A05
#define          CTS_TXSVAL1_IVCTX 0x00000A06
#define          CTS_TXSVAL2_IVCTX 0x00000A07
#define          CTS_TXSVAL3_IVCTX 0x00000A08
#define          CTS_TXHVAL1_IVCTX 0x00000A09
#define          CTS_TXHVAL2_IVCTX 0x00000A0A
#define          CTS_TXHVAL3_IVCTX 0x00000A0B
#define        ACR_CTS_CTRL1_IVCTX 0x00000A0C
#define        ACR_CTS_CTRL2_IVCTX 0x00000A0D
#define      ACR_CTS_CLK_DIV_IVCTX 0x00000A0E
#define       FS_HIGH_PERIOD_IVCTX 0x00000A0F
#define        FS_LOW_PERIOD_IVCTX 0x00000A10
#define        AIP_IN_AUD_ID_IVCTX 0x00000A12
#define               AUD_EN_IVCTX 0x00000A13
#define             AUD_MODE_IVCTX 0x00000A14
#define           SPDIF_CTRL_IVCTX 0x00000A15
#define          SPDIF_SSMPL_IVCTX 0x00000A16
#define          SPDIF_HSMPL_IVCTX 0x00000A17
#define             SPDIF_FS_IVCTX 0x00000A18
#define         SPDIF_SSMPL2_IVCTX 0x00000A19
#define         SPDIF_HSMPL2_IVCTX 0x00000A1A
#define           SPDIF_ERTH_IVCTX 0x00000A1B
#define           I2S_IN_MAP_IVCTX 0x00000A1C
#define          I2S_IN_CTRL_IVCTX 0x00000A1D
#define            I2S_CHST0_IVCTX 0x00000A1E
#define            I2S_CHST1_IVCTX 0x00000A1F
#define            I2S_CHST2_IVCTX 0x00000A20
#define            I2S_CHST3_IVCTX 0x00000A21
#define            I2S_CHST4_IVCTX 0x00000A22
#define            I2S_CHST5_IVCTX 0x00000A23
#define            I2S_CHST6_IVCTX 0x00000A24
#define                 ASRC_IVCTX 0x00000A25
#define          I2S_IN_SIZE_IVCTX 0x00000A26
#define     AUD_PAR_BUSCLK_1_IVCTX 0x00000A27
#define     AUD_PAR_BUSCLK_2_IVCTX 0x00000A28
#define     AUD_PAR_BUSCLK_3_IVCTX 0x00000A29
#define       DSD_INTERLEAVE_IVCTX 0x00000A2A
#define   AUDIO_PAR_MODE_SEL_IVCTX 0x00000A2B
#define              AIP_RST_IVCTX 0x00000A2C
#define         AIP_HDMI2MHL_IVCTX 0x00000A2D
#define             TDM_CTRL_IVCTX 0x00000A2E
#define          AUDP_TXCTRL_IVCTX 0x00000A2F
#define          AUDP_TXFIFO_IVCTX 0x00000A31
#define       AIP_REG_ACC_TO_IVCTX 0x00000A32
#define       AIP_TDM_CH_MAP_IVCTX 0x00000A33
#define       AIP_TDM_CTRL_4_IVCTX 0x00000A34
#define     AIP_EXT_TDM_CTRL_IVCTX 0x00000A35
#define AIP_TDM_TCF_AUTO_RST_IVCTX 0x00000A36
#define        AIP_3D_AUD_ON_IVCTX 0x00000A37
#define       AIP_3D_AUD_PID_IVCTX 0x00000A38
#define       AIP_INTR_STATE_IVCTX 0x00000A4E
#define            AIP_INTR1_IVCTX 0x00000A4F
#define            AIP_INTR2_IVCTX 0x00000A50
#define       AIP_INTR1_MASK_IVCTX 0x00000A51
#define       AIP_INTR2_MASK_IVCTX 0x00000A52
#define            RSVD1_API_IVCTX 0x00000A53
#define   TPI_DOWN_SMPL_CTRL_IVCTX 0x00000A61
#define       TPI_AUD_CONFIG_IVCTX 0x00000A62
#define           TPI_AUD_FS_IVCTX 0x00000A63
#define      AIP_TDM_CH_MAP2_IVCTX 0x00000A64
#define    AUD_TDM_CH_SWAP_1_IVCTX 0x00000A65
#define    AUD_TDM_CH_SWAP_2_IVCTX 0x00000A66
#define      AIP_TDM_CH_MAP3_IVCTX 0x00000A67
#define      AIP_TDM_CH_MAP4_IVCTX 0x00000A68
#define      AIP_TDM_CH_MAP5_IVCTX 0x00000A69
#define      AIP_TDM_CH_MAP6_IVCTX 0x00000A6A
#define      AIP_TDM_CH_MAP7_IVCTX 0x00000A6B
#define      AIP_TDM_CH_MAP8_IVCTX 0x00000A6C
#define       PRL_AUD_CH_NUM_IVCTX 0x00000A6D
#define        AUDIO_CLK_DIV_IVCTX 0x00000A6E

//====================VIDEO_PATH_CORE_REG===============
#define                                 VP_FEATURES_IVCTX 0x00000B00
#define                               VP_BUILD_TIME_IVCTX 0x00000B08
#define                               VP_SOFT_RESET_IVCTX 0x00000B0C
#define                          VP_DATA_BITS_VALUE_IVCTX 0x00000B0E
#define                               VP_INPUT_MUTE_IVCTX 0x00000B10
#define                        VP_INPUT_SYNC_CONFIG_IVCTX 0x00000B12
#define                             VP_INPUT_FORMAT_IVCTX 0x00000B14
#define                            VP_INPUT_MAPPING_IVCTX 0x00000B16
#define                               VP_INPUT_MASK_IVCTX 0x00000B18
#define                          VP_PIPELINE_CONFIG_IVCTX 0x00000B1A
#define                 VP_INPUT_SYNC_ADJUST_CONFIG_IVCTX 0x00000B1C
#define                             VP_DEGEN_CONFIG_IVCTX 0x00000B20
#define                        VP_DEGEN_PIXEL_DELAY_IVCTX 0x00000B22
#define              VP_DEGEN_PIXEL_COUNT_MINUS_ONE_IVCTX 0x00000B24
#define                         VP_DEGEN_LINE_DELAY_IVCTX 0x00000B26
#define                         VP_DEGEN_LINE_COUNT_IVCTX 0x00000B28
#define                            VP_DEC656_CONFIG_IVCTX 0x00000B30
#define         VP_DEC656_DELAY_EAV_TO_HSYNC_ACTIVE_IVCTX 0x00000B32
#define       VP_DEC656_PULSE_WIDTH_HSYNC_MINUS_ONE_IVCTX 0x00000B34
#define                 VP_DEC656_VSYNC_FRONT_PORCH_IVCTX 0x00000B36
#define           VP_DEC656_EAV_TO_VSYNC_DELAY_EVEN_IVCTX 0x00000B38
#define            VP_DEC656_EAV_TO_VSYNC_DELAY_ODD_IVCTX 0x00000B3A
#define                 VP_DEC656_PULSE_WIDTH_VSYNC_IVCTX 0x00000B3C
#define                            VP_DEC656_STATUS_IVCTX 0x00000B3E
#define                              VP_OUTPUT_MUTE_IVCTX 0x00000B40
#define                       VP_OUTPUT_SYNC_CONFIG_IVCTX 0x00000B42
#define                           VP_OUTPUT_MAPPING_IVCTX 0x00000B44
#define                              VP_OUTPUT_MASK_IVCTX 0x00000B46
#define                            VP_OUTPUT_FORMAT_IVCTX 0x00000B48
#define                  VP_OUTPUT_BLANK_START_LINE_IVCTX 0x00000B4C
#define                    VP_OUTPUT_BLANK_END_LINE_IVCTX 0x00000B4E
#define                      VP_OUTPUT_BLANK_CONFIG_IVCTX 0x00000B50
#define                   VP_OUTPUT_BLANKING_STATUS_IVCTX 0x00000B52
#define                           VP_OUTPUT_BLANK_Y_IVCTX 0x00000B54
#define                          VP_OUTPUT_BLANK_CB_IVCTX 0x00000B56
#define                          VP_OUTPUT_BLANK_CR_IVCTX 0x00000B58
#define                          VP_OUTPUT_ACTIVE_Y_IVCTX 0x00000B5A
#define                         VP_OUTPUT_ACTIVE_CB_IVCTX 0x00000B5C
#define                         VP_OUTPUT_ACTIVE_CR_IVCTX 0x00000B5E
#define                  VP_VTG_HORIZONTAL_SYNC_END_IVCTX 0x00000B60
#define        VP_VTG_HORIZONTAL_ACTIVE_VIDEO_START_IVCTX 0x00000B62
#define                             VP_VTG_HALFLINE_IVCTX 0x00000B64
#define          VP_VTG_HORIZONTAL_ACTIVE_VIDEO_END_IVCTX 0x00000B66
#define                          VP_VTG_END_OF_LINE_IVCTX 0x00000B68
#define                    VP_VTG_VERTICAL_SYNC_END_IVCTX 0x00000B70
#define                        VP_VTG_TRIGGER_START_IVCTX 0x00000B72
#define          VP_VTG_VERTICAL_ACTIVE_VIDEO_START_IVCTX 0x00000B74
#define            VP_VTG_VERTICAL_ACTIVE_VIDEO_END_IVCTX 0x00000B76
#define                VP_VTG_VERTICAL_END_OF_FRAME_IVCTX 0x00000B78
#define                               VP_VTG_CONFIG_IVCTX 0x00000B7A
#define                            VP_VTG_THRESHOLD_IVCTX 0x00000B7B
#define                          VP_VTG_CYCLE_DELAY_IVCTX 0x00000B7C
#define                       VP_VTG_UPDATE_REQUEST_IVCTX 0x00000B7E
#define                          VP_VTG_BANK_CONFIG_IVCTX 0x00000B7F
#define                              VP_FDET_CONFIG_IVCTX 0x00000B80
#define                              VP_FDET_STATUS_IVCTX 0x00000B81
#define                               VP_FDET_CLEAR_IVCTX 0x00000B82
#define                 VP_FDET_INTERLACE_THRESHOLD_IVCTX 0x00000B83
#define          VP_FDET_FRAME_RATE_DELTA_THRESHOLD_IVCTX 0x00000B84
#define                          VP_FDET_FRAME_RATE_IVCTX 0x00000B88
#define                         VP_FDET_PIXEL_COUNT_IVCTX 0x00000B8C
#define                          VP_FDET_LINE_COUNT_IVCTX 0x00000B8E
#define                     VP_FDET_HSYNC_LOW_COUNT_IVCTX 0x00000B90
#define                    VP_FDET_HSYNC_HIGH_COUNT_IVCTX 0x00000B92
#define                        VP_FDET_HFRONT_COUNT_IVCTX 0x00000B94
#define                         VP_FDET_HBACK_COUNT_IVCTX 0x00000B96
#define                VP_FDET_VSYNC_LOW_COUNT_EVEN_IVCTX 0x00000B98
#define               VP_FDET_VSYNC_HIGH_COUNT_EVEN_IVCTX 0x00000B9A
#define                   VP_FDET_VFRONT_COUNT_EVEN_IVCTX 0x00000B9C
#define                    VP_FDET_VBACK_COUNT_EVEN_IVCTX 0x00000B9E
#define                 VP_FDET_VSYNC_LOW_COUNT_ODD_IVCTX 0x00000BA0
#define                VP_FDET_VSYNC_HIGH_COUNT_ODD_IVCTX 0x00000BA2
#define                    VP_FDET_VFRONT_COUNT_ODD_IVCTX 0x00000BA4
#define                     VP_FDET_VBACK_COUNT_ODD_IVCTX 0x00000BA6
#define                         VP_FDET_FRAME_COUNT_IVCTX 0x00000BA8
#define           VP_FDET_LINE_RATE_DELTA_THRESHOLD_IVCTX 0x00000BAA
#define                           VP_FDET_LINE_RATE_IVCTX 0x00000BAC
#define             VP_FDET_VSYNC_HSYNC_OFFSET_EVEN_IVCTX 0x00000BB0
#define              VP_FDET_VSYNC_HSYNC_OFFSET_ODD_IVCTX 0x00000BB2
#define                            VP_FDET_IRQ_MASK_IVCTX 0x00000BB8
#define                          VP_FDET_IRQ_STATUS_IVCTX 0x00000BBC
#define                     VP_EMBD_SYNC_ENC_CONFIG_IVCTX 0x00000BC0
#define               VP_EMBD_SYNC_ENC_SECT0_CONFIG_IVCTX 0x00000BC4
#define               VP_EMBD_SYNC_ENC_SECT1_CONFIG_IVCTX 0x00000BC8
#define               VP_EMBD_SYNC_ENC_SECT2_CONFIG_IVCTX 0x00000BCC
#define               VP_EMBD_SYNC_ENC_SECT3_CONFIG_IVCTX 0x00000BD0
#define               VP_EMBD_SYNC_ENC_SECT4_CONFIG_IVCTX 0x00000BD4
#define               VP_EMBD_SYNC_ENC_SECT5_CONFIG_IVCTX 0x00000BD8
#define     VP_EMBD_SYNC_ENC_HORIZONTAL_DATA_PIXELS_IVCTX 0x00000BDC
#define VP_EMBD_SYNC_ENC_HORIZONTAL_BLANKING_PIXELS_IVCTX 0x00000BDE
#define             VP_EMBD_SYNC_ENC_UPDATE_REQUEST_IVCTX 0x00000BE0
#define                VP_EMBD_SYNC_ENC_BANK_CONFIG_IVCTX 0x00000BE1

//====================VIDEO_PATH_CORE_CMS_CSC0_REG=============
#define                 VP_CMS_CSC0_FEATURES_IVCTX 0x00000D00
#define          VP_CMS_CSC0_DATA_BITS_VALUE_IVCTX 0x00000D02
#define               VP_CMS_CSC0_BUILD_TIME_IVCTX 0x00000D04
#define         VP_CMS_CSC0_C420_C422_CONFIG_IVCTX 0x00000D08
#define         VP_CMS_CSC0_C422_C444_CONFIG_IVCTX 0x00000D0C
#define         VP_CMS_CSC0_PIXCAP_IN_CONFIG_IVCTX 0x00000D10
#define        VP_CMS_CSC0_PIXCAP_IN_CONTROL_IVCTX 0x00000D11
#define         VP_CMS_CSC0_PIXCAP_IN_STATUS_IVCTX 0x00000D12
#define          VP_CMS_CSC0_PIXCAP_IN_PIXEL_IVCTX 0x00000D14
#define           VP_CMS_CSC0_PIXCAP_IN_LINE_IVCTX 0x00000D16
#define              VP_CMS_CSC0_PIXCAP_IN_Y_IVCTX 0x00000D18
#define             VP_CMS_CSC0_PIXCAP_IN_CB_IVCTX 0x00000D1A
#define             VP_CMS_CSC0_PIXCAP_IN_CR_IVCTX 0x00000D1C
#define         VP_CMS_CSC0_MULTI_CSC_CONFIG_IVCTX 0x00000D20
#define VP_CMS_CSC0_MULTI_CSC_MULTCOEFF_ADDR_IVCTX 0x00000D22
#define VP_CMS_CSC0_MULTI_CSC_MULTCOEFF_DATA_IVCTX 0x00000D24
#define   VP_CMS_CSC0_MULTI_CSC_IN_GY_OFFSET_IVCTX 0x00000D28
#define  VP_CMS_CSC0_MULTI_CSC_IN_BCB_OFFSET_IVCTX 0x00000D2C
#define  VP_CMS_CSC0_MULTI_CSC_IN_RCR_OFFSET_IVCTX 0x00000D30
#define  VP_CMS_CSC0_MULTI_CSC_OUT_GY_OFFSET_IVCTX 0x00000D34
#define VP_CMS_CSC0_MULTI_CSC_OUT_BCB_OFFSET_IVCTX 0x00000D38
#define VP_CMS_CSC0_MULTI_CSC_OUT_RCR_OFFSET_IVCTX 0x00000D3C

//====================VIDEO_PATH_CORE_CMS_CSC1_REG=============
#define                 VP_CMS_CSC1_FEATURES_IVCTX 0x00000D80
#define          VP_CMS_CSC1_DATA_BITS_VALUE_IVCTX 0x00000D82
#define               VP_CMS_CSC1_BUILD_TIME_IVCTX 0x00000D84
#define         VP_CMS_CSC1_MULTI_CSC_CONFIG_IVCTX 0x00000DA0
#define VP_CMS_CSC1_MULTI_CSC_MULTCOEFF_ADDR_IVCTX 0x00000DA2
#define VP_CMS_CSC1_MULTI_CSC_MULTCOEFF_DATA_IVCTX 0x00000DA4
#define   VP_CMS_CSC1_MULTI_CSC_IN_GY_OFFSET_IVCTX 0x00000DA8
#define  VP_CMS_CSC1_MULTI_CSC_IN_BCB_OFFSET_IVCTX 0x00000DAC
#define  VP_CMS_CSC1_MULTI_CSC_IN_RCR_OFFSET_IVCTX 0x00000DB0
#define  VP_CMS_CSC1_MULTI_CSC_OUT_GY_OFFSET_IVCTX 0x00000DB4
#define VP_CMS_CSC1_MULTI_CSC_OUT_BCB_OFFSET_IVCTX 0x00000DB8
#define VP_CMS_CSC1_MULTI_CSC_OUT_RCR_OFFSET_IVCTX 0x00000DBC
#define         VP_CMS_CSC1_C444_C422_CONFIG_IVCTX 0x00000DC0
#define         VP_CMS_CSC1_C422_C420_CONFIG_IVCTX 0x00000DC2
#define            VP_CMS_CSC1_DITHER_CONFIG_IVCTX 0x00000DC4
#define        VP_CMS_CSC1_RANGE_CLIP_CONFIG_IVCTX 0x00000DC6
#define         VP_CMS_CSC1_RANGE_CLIP_Y_MIN_IVCTX 0x00000DC8
#define         VP_CMS_CSC1_RANGE_CLIP_Y_MAX_IVCTX 0x00000DCA
#define         VP_CMS_CSC1_RANGE_CLIP_C_MIN_IVCTX 0x00000DCC
#define         VP_CMS_CSC1_RANGE_CLIP_C_MAX_IVCTX 0x00000DCE
#define        VP_CMS_CSC1_PIXCAP_OUT_CONFIG_IVCTX 0x00000DD0
#define       VP_CMS_CSC1_PIXCAP_OUT_CONTROL_IVCTX 0x00000DD1
#define        VP_CMS_CSC1_PIXCAP_OUT_STATUS_IVCTX 0x00000DD2
#define         VP_CMS_CSC1_PIXCAP_OUT_PIXEL_IVCTX 0x00000DD4
#define          VP_CMS_CSC1_PIXCAP_OUT_LINE_IVCTX 0x00000DD6
#define             VP_CMS_CSC1_PIXCAP_OUT_Y_IVCTX 0x00000DD8
#define            VP_CMS_CSC1_PIXCAP_OUT_CB_IVCTX 0x00000DDA
#define            VP_CMS_CSC1_PIXCAP_OUT_CR_IVCTX 0x00000DDC

//====================HDR_REG===============
#define          D_HDR_GEN_CTL_IVCTX 0x00000F00
#define          D_HDR_INTRSTS_IVCTX 0x00000F01
#define         D_HDR_INTRMASK_IVCTX 0x00000F02
#define           D_HDR_PC_HB0_IVCTX 0x00000F03
#define           D_HDR_PC_HB1_IVCTX 0x00000F04
#define           D_HDR_PC_PB3_IVCTX 0x00000F05
#define           D_HDR_PC_PB4_IVCTX 0x00000F06
#define           D_HDR_PC_PB5_IVCTX 0x00000F07
#define           D_HDR_EM_HB0_IVCTX 0x00000F08
#define           D_HDR_EM_PB0_IVCTX 0x00000F09
#define           D_HDR_EM_PB2_IVCTX 0x00000F0A
#define           D_HDR_EM_PB3_IVCTX 0x00000F0B
#define           D_HDR_EM_PB4_IVCTX 0x00000F0C
#define        D_HDR_ACK_TO_HI_IVCTX 0x00000F0D
#define        D_HDR_ACK_TO_LO_IVCTX 0x00000F0E
#define        D_HDR_ACT_DE_HI_IVCTX 0x00000F0F
#define        D_HDR_ACT_DE_LO_IVCTX 0x00000F10
#define            D_HDR_VB_LE_IVCTX 0x00000F11
#define        D_HDR_READ_PORT_IVCTX 0x00000F12
#define              D_HDR_DBG_IVCTX 0x00000F13
#define        D_HDR_MEM_WDATA_IVCTX 0x00000F14
#define      D_HDR_INSERT_CTRL_IVCTX 0x00000F15
#define D_HDR_INSERT_PAYLOAD_0_IVCTX 0x00000F16
#define D_HDR_INSERT_PAYLOAD_1_IVCTX 0x00000F17
#define    D_HDR_MEM_WADDR_RST_IVCTX 0x00000F18
#define     D_HDR_INSERT_CTL_1_IVCTX 0x00000F19
#define     D_HDR_INSERT_CTL_2_IVCTX 0x00000F1A
#define     D_HDR_INSERT_CTL_3_IVCTX 0x00000F1B
#define       D_HDR_FAPA_DEBUG_IVCTX 0x00000F1C
#define          D_HDR_SPARE_0_IVCTX 0x00000F1D
#define          D_HDR_SPARE_1_IVCTX 0x00000F1E
#define          D_HDR_SPARE_2_IVCTX 0x00000F1F
#define          D_HDR_SPARE_3_IVCTX 0x00000F20
#define          D_HDR_SPARE_4_IVCTX 0x00000F21
#define          D_HDR_SPARE_5_IVCTX 0x00000F22
#define          D_HDR_SPARE_6_IVCTX 0x00000F23
#define          D_HDR_SPARE_7_IVCTX 0x00000F24
#define          D_HDR_SPARE_8_IVCTX 0x00000F25
#define          D_HDR_SPARE_9_IVCTX 0x00000F26

//====================DSC_PKT_REG===============
#define          DSC_PKT_GEN_CTL_IVCTX 0x00000F80
#define          DSC_PKT_INTRSTS_IVCTX 0x00000F81
#define         DSC_PKT_INTRMASK_IVCTX 0x00000F82
#define           DSC_PKT_PC_HB0_IVCTX 0x00000F83
#define           DSC_PKT_PC_HB1_IVCTX 0x00000F84
#define           DSC_PKT_PC_PB3_IVCTX 0x00000F85
#define           DSC_PKT_PC_PB4_IVCTX 0x00000F86
#define           DSC_PKT_PC_PB5_IVCTX 0x00000F87
#define           DSC_PKT_EM_HB0_IVCTX 0x00000F88
#define           DSC_PKT_EM_PB0_IVCTX 0x00000F89
#define           DSC_PKT_EM_PB2_IVCTX 0x00000F8A
#define           DSC_PKT_EM_PB3_IVCTX 0x00000F8B
#define           DSC_PKT_EM_PB4_IVCTX 0x00000F8C
#define        DSC_PKT_ACK_TO_HI_IVCTX 0x00000F8D
#define        DSC_PKT_ACK_TO_LO_IVCTX 0x00000F8E
#define        DSC_PKT_ACT_DE_HI_IVCTX 0x00000F8F
#define        DSC_PKT_ACT_DE_LO_IVCTX 0x00000F90
#define            DSC_PKT_VB_LE_IVCTX 0x00000F91
#define        DSC_PKT_READ_PORT_IVCTX 0x00000F92
#define              DSC_PKT_DBG_IVCTX 0x00000F93
#define        DSC_PKT_MEM_WDATA_IVCTX 0x00000F94
#define      DSC_PKT_INSERT_CTRL_IVCTX 0x00000F95
#define    DSC_PKT_MEM_WADDR_RST_IVCTX 0x00000F96
#define     DSC_PKT_INSERT_CTL_1_IVCTX 0x00000F97
#define DSC_PKT_INSERT_PAYLOAD_0_IVCTX 0x00000F98
#define DSC_PKT_INSERT_PAYLOAD_1_IVCTX 0x00000F99
#define     DSC_PKT_INSERT_CTL_2_IVCTX 0x00000F9A
#define     DSC_PKT_INSERT_CTL_3_IVCTX 0x00000F9B
#define     DSC_PKT_INSERT_CTL_4_IVCTX 0x00000F9C
#define     DSC_PKT_INSERT_CTL_5_IVCTX 0x00000F9D
#define    DSC_PKT_MAX_PAYLOAD_0_IVCTX 0x00000F9E
#define    DSC_PKT_MAX_PAYLOAD_1_IVCTX 0x00000F9F
#define          DSC_PKT_SPARE_0_IVCTX 0x00000FA0
#define          DSC_PKT_SPARE_1_IVCTX 0x00000FA1
#define          DSC_PKT_SPARE_2_IVCTX 0x00000FA2
#define          DSC_PKT_SPARE_3_IVCTX 0x00000FA3
#define          DSC_PKT_SPARE_4_IVCTX 0x00000FA4
#define          DSC_PKT_SPARE_5_IVCTX 0x00000FA5
#define          DSC_PKT_SPARE_6_IVCTX 0x00000FA6
#define          DSC_PKT_SPARE_7_IVCTX 0x00000FA7
#define          DSC_PKT_SPARE_8_IVCTX 0x00000FA8
#define          DSC_PKT_SPARE_9_IVCTX 0x00000FA9

//====================ZONEVCO_REG===============
#define      ZONE_CTRL_0_IVCTX 0x00001000
#define    ZONE_STATUS_0_IVCTX 0x00001001
#define          DEBUG_0_IVCTX 0x00001002
#define          DEBUG_1_IVCTX 0x00001003
#define    VCOCAL_CTRL_0_IVCTX 0x00001004
#define  VCOCAL_STATUS_0_IVCTX 0x00001005
#define PRECLK_FORCE_CTL_IVCTX 0x00001006
#define   PRECLK_CNT_MAX_IVCTX 0x00001007
#define     OCLK_CNT_MAX_IVCTX 0x00001008
#define     OCLK_CNT_MIN_IVCTX 0x00001009
#define          PRE_DIV_IVCTX 0x0000100A
#define         POST_DIV_IVCTX 0x0000100B
#define  VCOCAL_STATUS_1_IVCTX 0x0000100C
#define        PLL_MODE0_IVCTX 0x0000100D
#define         SZONE_OW_IVCTX 0x0000100E
#define          ISEL_OW_IVCTX 0x0000100F
#define       BIAS_BGR_D_IVCTX 0x00001010
#define       DIV_PRE_OW_IVCTX 0x00001011
#define       DIV_PST_OW_IVCTX 0x00001012
#define        VCOCAL_OW_IVCTX 0x00001013
#define        ZONE_INTR_IVCTX 0x00001014
#define   ZONE_INTR_MASK_IVCTX 0x00001015
#define      ZONE_CTRL_1_IVCTX 0x00001016
#define      ZONE_CTRL_2_IVCTX 0x00001017
#define       PLL_LK_DLY_IVCTX 0x00001018
#define      LKDT_CTRL_0_IVCTX 0x00001019
#define       LKDT_CTRL1_IVCTX 0x0000101A
#define       LKDT_CTRL2_IVCTX 0x0000101B
#define       LKDT_CTRL3_IVCTX 0x0000101C
#define     OCLK_CNT_THR_IVCTX 0x0000101D
#define      ZVTIMER_MAX_IVCTX 0x0000101E
#define          ZVSTATE_IVCTX 0x0000101F
#define       VCOCAL_ADJ_IVCTX 0x00001020
#define    ZONECAL_ADJ_0_IVCTX 0x00001021
#define    ZONECAL_ADJ_1_IVCTX 0x00001022
#define    ZONECAL_ADJ_2_IVCTX 0x00001023
#define    ZONECAL_ADJ_3_IVCTX 0x00001024
#define       ZONECTRL_3_IVCTX 0x00001025
#define       ZONECTRL_4_IVCTX 0x00001026
#define       ZONECTRL_5_IVCTX 0x00001027
#define       ZONECTRL_6_IVCTX 0x00001028
#define       ZONECTRL_7_IVCTX 0x00001029
#define       ZONECTRL_8_IVCTX 0x0000102A
#define       ZONECTRL_9_IVCTX 0x0000102B
#define       ZONECTRL_A_IVCTX 0x0000102C
#define       ZONECTRL_B_IVCTX 0x0000102D
#define       ZONECTRL_C_IVCTX 0x0000102E
#define       ZONECTRL_D_IVCTX 0x0000102F
#define       ZONECTRL_E_IVCTX 0x00001030
#define       ZONECTRL_F_IVCTX 0x00001031
#define    ZONE_STATUS_1_IVCTX 0x00001032
#define    ZONE_STATUS_2_IVCTX 0x00001033
#define    ZONE_STATUS_3_IVCTX 0x00001034
#define    ZONE_STATUS_4_IVCTX 0x00001035
#define    ZONE_STATUS_5_IVCTX 0x00001036
#define    ZONE_STATUS_6_IVCTX 0x00001037
#define    ZONE_STATUS_7_IVCTX 0x00001038
#define    ZONE_STATUS_8_IVCTX 0x00001039
#define    ZONE_STATUS_9_IVCTX 0x0000103A
#define    ZONE_STATUS_A_IVCTX 0x0000103B
#define     ZONE_CTRL_10_IVCTX 0x0000103C

#endif  /* __HDMI_TX_REG_H_ */
